@@ -8,10 +8,11 @@ import json
 
 from dptools.cli import BaseCLI
 
-# TODO: figure out what to do about type_map
-type_map = {"1": "O", "2": "Si", "3": "H"}
-# type_map = {'1': 'C', '2': 'H', '3': 'H', '4': 'O', '5': 'O', '6': 'H', '7': 'Cu'}
-def read_dump(dump, index=":"):
+def read_dump(dump, type_map, index=":"):
+    type_map = read_type_map(type_map) # support str, json, dict inputs
+    if 0 in type_map.values(): # lammps indexing starts at 1
+        type_map = {k: v + 1 for k, v in type_map.items()}
+    type_map = {v: k for k, v in type_map.items()} # invert to find symbol from type index
     with open(dump) as file:
         lines = file.readlines()
     traj = []
@@ -28,7 +29,7 @@ def read_dump(dump, index=":"):
             types = np.array(
                 [int(l.split()[-4]) for l in lines[i + 1 : i + 1 + n_atoms]]
             )
-            symbols = np.array([type_map[str(t)] for t in types])
+            symbols = np.array([type_map[t] for t in types])
             positions = np.array(
                 [str_to_float(lines[i + j + 1]) for j in range(n_atoms)]
             )
@@ -82,30 +83,56 @@ def graph2typemap(graph):
     type_map = {sym: i for i, sym in enumerate(dp.get_type_map())}
     return type_map
 
-def get_dpfaults():
-    """ like defaults but for dp (haha... ha..) """
-    graph = os.environ.get("DPTOOLS_MODEL", "./graph.pb")
-    type_map = os.environ.get("DPTOOLS_TYPE_MAP", None)
-    if not type_map and os.path.exists(graph):
-        type_map = graph2typemap(graph)
-    return graph, type_map
-
 def read_type_map(type_map_json):
-    with open(type_map_json) as file:
-        type_map = json.loads(file.read())
+    if isinstance(type_map_json, dict):
+        type_map = type_map_json
+    elif isinstance(type_map_json, str):
+        if type_map_json.endswith(".json"):
+            with open(type_map_json) as file:
+                type_map = json.loads(file.read())
+        else:
+            type_map = str2typemap(type_map_json)
+    else:
+        raise TypeError(f"Unknown type_map format provided: {type_map_json}")
+    return check_type_map(type_map)
 
-    invert = False
-    # I'm inconsistent with key-value/value-key when writing type_map.json
+def check_type_map(type_map_dict):
+    # I'm inconsistent with key-value/value-key when writing type_map
     # so check if need to invert dict such that chem symbols are keys
     # TODO: fix inconsistencies
-    for k, v in type_map.items():
+    invert = False
+    for k, v in type_map_dict.items():
         if k not in chemical_symbols:
             invert = True
         break
     if invert:
-        type_map = {v: k for k, v in type_map.items()}
+        type_map_dict = {v: k for k, v in type_map_dict.items()}
+    return type_map_dict
+
+def typemap2str(type_map_dict):
+    type_map = check_type_map(type_map_dict)
+    tm_str = ",".join([f"{k}:{v}" for k, v in type_map.items()])
+    return tm_str
+
+def str2typemap(tm_str):
+    items = tm_str.split(",")
+    type_map = {i.split(":")[0]: int(i.split(":")[-1]) for i in items}
     return type_map
 
+def randomize_seed(in_json):
+    if isinstance(in_json, str):
+        with open(in_json) as file:
+            in_json = json.loads(file.read())
+    elif not isinstance(in_json, dict):
+        raise TypeError("Need dict or .json file to randomize seeds")
+    in_json["model"]["descriptor"]["seed"] = get_seed()
+    in_json["model"]["fitting_net"]["seed"] = get_seed()
+    in_json["training"]["seed"] = get_seed()
+    return in_json
+
+def get_seed(max_val=999999):
+    return np.random.randint(max_val)
+
 class Converter:
     def __init__(self, inputs, output, indices=":"):
         self.inputs = inputs
